// SPDX-FileCopyrightText: 2011 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef OVERFLOWWIDGET_H
#define OVERFLOWWIDGET_H

#include <QWidget>
#include <QScrollArea>
#include <QBoxLayout>
#include <QPushButton>

#include <DBlurEffectWidget>

#include "dockitem.h"

DWIDGET_USE_NAMESPACE

class IconButton;
class MaskWidget;

class OverflowWidget : public DBlurEffectWidget
{
    Q_OBJECT

public:
    explicit OverflowWidget(QWidget *parent = nullptr);

    int itemCount() const;
    void setSize(int width, int height, int itemSize);
    void addItem(QWidget *item);
    void setLayoutPosition(Dock::Position position);
    QWidget *getFirstItem() const;
    void setAllRadius(int radius);
    const QList<DockItem *> allItem() { return m_centerScroll->findChildren<DockItem *>(); }

public slots:
    void setAllMaskAlpha(quint8 alpha);

protected:
    bool eventFilter(QObject *watched, QEvent *event) override;

private:
    void initUI();
    void initConnections();
    void setMaskVisible();
    void setMaskShape();
    int getJumpStep() const;
    void hideItemPopup();

private slots:
    void leftBtnScrollAction();
    void rightBtnScrollAction();

private:
    QBoxLayout *m_mainLayout;

    QScrollArea *m_scrollArea;
    DBlurEffectWidget *m_centerScroll;
    QBoxLayout *m_scrollLayout;

    MaskWidget *m_leftMask;
    MaskWidget *m_rightMask;
    QHBoxLayout *m_leftMaskLayout;
    QHBoxLayout *m_rightMaskLayout;

    IconButton *m_leftBtn;
    IconButton *m_rightBtn;
};

class MaskWidget : public DBlurEffectWidget
{
    Q_OBJECT

public:
    explicit MaskWidget(QWidget *parent = nullptr) : DBlurEffectWidget(parent) {}

    enum RadiusPos { Left = 0, Right, Top, Bottom };

    void setRadiusPos(const RadiusPos &pos);

protected:
    void paintEvent(QPaintEvent *e) override;

private:
    RadiusPos m_radiusPos;
};

class IconButton : public QPushButton
{
    Q_OBJECT

public:
    explicit IconButton(QWidget *parent = nullptr);

    void setIconPath(const QString &path)
    {
        if (!path.isEmpty()) {
            setIcon(QIcon(path));
        }
    }

protected:
    void paintEvent(QPaintEvent *e) override;
};

#endif // OVERFLOWWIDGET_H
