// SPDX-FileCopyrightText: 2011 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef OVERFLOWITEM_H
#define OVERFLOWITEM_H

#include "dockitem.h"
#include "overflowwidget.h"

#include <DWidget>
#include <DRegionMonitor>

QT_USE_NAMESPACE
class QScrollArea;
class QPushButton;
class QBoxLayout;

DWIDGET_BEGIN_NAMESPACE
class DIconButton;
DWIDGET_END_NAMESPACE

DWIDGET_USE_NAMESPACE

namespace Dock {
class TipsWidget;
}

class OverflowItem : public DockItem
{
    Q_OBJECT

public:
    explicit OverflowItem(QWidget *parent = nullptr);
    ~OverflowItem() override;
    inline ItemType itemType() const override { return Overflow; }
    int itemCountOnZone() const;
    void setPopUpSize(int width, int height, QSize panelSize, QPoint panelPos, int itemSize);
    void addItem(QWidget *item);
    void hidePopUpWindow();
    void setLayoutPosition(Dock::Position position);
    void setLayoutMargin(int margin);

public slots:
    void refreshIcon() override;

signals:
    void maskAlphaChanged(quint8 alpha);
    void popupWindowCursorOut();
    void popupWindowShowChanged(bool show, const QRect &rect);

private:
    bool eventFilter(QObject *watched, QEvent *event) override;
    void mousePressEvent(QMouseEvent *e) override;
    bool event(QEvent *event) override;
    void paintEvent(QPaintEvent *e) override;
    void showOverflowPopupWindow(QWidget *const content, const bool model = false, const int radius = 6);
    QWidget *popupTips() override;
    QPoint calcPopupWindowCenterPos();
    QPoint overflowIconPosition(const QPixmap &pixmap) const;
    void initUI();
    void initConnections();
    QRect calcPopupWindowRect();
    void setItemTipSwitch(bool needTip);
    void updateItemMinimizedGeometry(bool needUpdate);

private slots:
    void onGlobMouseRelease(const QPoint &mousePos, const int flag);

private:
    bool m_showPopup;
    QSize m_panelSize;
    QPixmap m_overflowIcon;
    QPoint m_panelPos;
    QPixmap m_moreIcon;
    int m_popupMargin;  // 溢出区弹窗与屏幕的左右间距

    DockPopupWindow *m_popupWindow;
    OverflowWidget *m_contentWidget;

    DRegionMonitor *m_regionInter;
    TipsWidget *m_tips;

    QEvent::Type m_state;
};

#endif // !OVERFLOWITEM_H
