// SPDX-FileCopyrightText: 2011 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#include "settingmanager.h"

#include <DConfig>

DCORE_USE_NAMESPACE

SettingManager *SettingManager::instance()
{
    static SettingManager config;
    return &config;
}

bool SettingManager::enableSafeMode() const
{
    return m_enableSafeMode;
}

int SettingManager::delayIntervalOnHide() const
{
    return m_delayIntervalOnHide;
}

int SettingManager::showWindowName() const
{
    return m_showWindowName;
}

int SettingManager::toggleDesktopInterval() const
{
    return m_toggleDesktopInterval;
}

bool SettingManager::alwaysHideDock() const
{
    return m_alwaysHideDock;
}

SettingManager::SettingManager(QObject *parent)
    : QObject(parent)
    , m_enableSafeMode(true)
    , m_delayIntervalOnHide(0)
    , m_showWindowName(-1)
    , m_toggleDesktopInterval(0)
    , m_alwaysHideDock(false)
{
    DConfig *dockConfig = DConfig::create("org.deepin.dde.dock", "org.deepin.dde.dock", QString(), this);

    if (dockConfig) {
        connect(dockConfig, &DConfig::valueChanged, this, &SettingManager::onDockConfigChanged);

        QStringList keys = dockConfig->keyList();
        if (keys.contains("enableSafeMode")) {
            m_enableSafeMode = dockConfig->value("enableSafeMode").toBool();
        }
        if (keys.contains("delayIntervalOnHide")) {
            m_delayIntervalOnHide = dockConfig->value("delayIntervalOnHide").toInt();
        }
        if (keys.contains("showWindowName")) {
            m_showWindowName = dockConfig->value("showWindowName").toInt();
        }
        if (keys.contains("toggleDesktopInterval")) {
            m_toggleDesktopInterval = dockConfig->value("toggleDesktopInterval").toInt();
        }
        if (keys.contains("alwaysHideDock")) {
            m_alwaysHideDock = dockConfig->value("alwaysHideDock").toBool();
        }
    }
}

SettingManager::~SettingManager()
{
}

void SettingManager::onDockConfigChanged(const QString &key)
{
    DConfig *dockConfig = static_cast<DConfig *>(sender());
    if (key == "enableSafeMode") {
        m_enableSafeMode = dockConfig->value("enableSafeMode").toBool();
    } else if (key == "delayIntervalOnHide") {
        m_delayIntervalOnHide = dockConfig->value("delayIntervalOnHide").toInt();
    } else if (key == "showWindowName") {
        m_showWindowName = dockConfig->value("showWindowName").toInt();
    } else if (key == "toggleDesktopInterval") {
        m_toggleDesktopInterval = dockConfig->value("toggleDesktopInterval").toInt();
        emit toggleDesktopIntervalChanged(m_toggleDesktopInterval);
    } else if (key == "alwaysHideDock") {
        m_alwaysHideDock = dockConfig->value("alwaysHideDock").toBool();
        emit alwaysHideDockChanged(m_alwaysHideDock);
    }
}
