// SPDX-FileCopyrightText: 2011 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef SETTINGMANAGER_H
#define SETTINGMANAGER_H

#include <QObject>

/**
 * @brief 该类将DConfig的配置都移动到单独的类中进行管理，防止在多处重复创建DConfig对象时间过长
 */

class SettingManager : public QObject
{
    Q_OBJECT

public:
    static SettingManager *instance();
    SettingManager(const SettingManager &) = delete;
    SettingManager &operator=(const SettingManager &) = delete;

    bool enableSafeMode() const;
    int delayIntervalOnHide() const;
    int showWindowName() const;
    int toggleDesktopInterval() const;
    bool alwaysHideDock() const;

signals:
    void onlyShowPrimaryChanged(bool);
    void alwaysHideDockChanged(bool);
    void toggleDesktopIntervalChanged(int);

protected:
    explicit SettingManager(QObject *parent = nullptr);
    ~SettingManager();

private slots:
    void onDockConfigChanged(const QString &key);

private:
    bool m_enableSafeMode;
    int m_delayIntervalOnHide;
    int m_showWindowName;
    int m_toggleDesktopInterval;
    bool m_alwaysHideDock;
};

#endif // CONFIGWATCHER_H
