// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef MAINPANELCONTROL_H
#define MAINPANELCONTROL_H

#include "constants.h"
#include "appitem.h"

#include <DWidget>

#include <com_deepin_daemon_gesture.h>
#include <QTimer>
using namespace Dock;

using DBusDock = com::deepin::dde::daemon::Dock;

DWIDGET_BEGIN_NAMESPACE
class DIconButton;
DWIDGET_END_NAMESPACE

DWIDGET_USE_NAMESPACE

class QScrollArea;
class QBoxLayout;
class QLabel;
class TrayPluginItem;
class PluginsItem;
class DockItem;
class PlaceholderItem;
class AppDragWidget;
class DesktopWidget;
class OverflowItem;

class MainPanelControl : public QWidget
{
    Q_OBJECT
public:
    explicit MainPanelControl(QWidget *parent = nullptr);
    ~MainPanelControl() override;

    void setPositonValue(Position position);
    void setDisplayMode(DisplayMode dislayMode);
    void resizeDockIcon();
    void setToggleDesktopInterval(int ms);
    void hideOverflowPopupWindow();

public slots:
    void insertItem(const int index, DockItem *item);
    void removeItem(DockItem *item);
    void itemUpdated(DockItem *item);
    void setKwinAppItemMinimizedGeometry(DockItem *item, const QRect);
    void updatePluginsLayout();

signals:
    void itemMoved(DockItem *sourceItem, DockItem *targetItem);
    void itemAdded(const QString &appDesktop, int idx);
    void requestUpdateLayout(bool needAdd, bool forceShow = true);
    void maskAlphaChanged(quint8 alpha);
    void overflowPopupCursorOut();
    void overflowPopupShowChanged(bool show, const QRect &rect);
    void requestOverflowRecentlyUsedAppAutoHide(bool autoHide);

private:
    void initUI();
    void updateAppAreaSonWidgetSize();
    void updateMainPanelLayout();
    void updateDisplayMode();
    void moveAppSonWidget();

    void addFixedAreaItem(int index, QWidget *wdg);
    void removeFixedAreaItem(QWidget *wdg);
    void addAppAreaItem(int index, QWidget *wdg);
    void removeAppAreaItem(QWidget *wdg);
    void addTrayAreaItem(int index, QWidget *wdg);
    void removeTrayAreaItem(QWidget *wdg);
    void addPluginAreaItem(int index, QWidget *wdg);
    void removePluginAreaItem(QWidget *wdg);

    // 拖拽相关
    void startDrag(DockItem *);
    DockItem *dropTargetItem(DockItem *sourceItem, QPoint point);
    void moveItem(DockItem *sourceItem, DockItem *targetItem);
    void handleDragMove(QDragMoveEvent *e, bool isFilter);
    void calcuDockIconSize(int appItemSize, int maxCount ,bool showOverflow, int traySize);
    void resizeDesktopWidget();
    bool checkNeedShowDesktop();
    bool appIsOnDock(const QString &appDesktop);

    int getItemIndex(DockItem *targetItem) const;

    void updateLayout(bool needAdd = true, bool forceShow = true);
    void updateOverflowRecentlyUsedApp(const AppItem *app);
    void setOverflowPopUpSize(int itemCount);

protected:
    void dragMoveEvent(QDragMoveEvent *e) override;
    void dragEnterEvent(QDragEnterEvent *e) override;
    void dragLeaveEvent(QDragLeaveEvent *e) override;
    void dropEvent(QDropEvent *) override;
    bool eventFilter(QObject *watched, QEvent *event) override;
    void enterEvent(QEvent *event) override;
    void mousePressEvent(QMouseEvent *e) override;
    void resizeEvent(QResizeEvent *event) override;
    void paintEvent(QPaintEvent *event) override;


private:
    // state
    int m_maxAppStayCount;           // 最多驻留的app数量
    bool m_showOverflow;             // 是否显示overflow区域
    int m_appItemSize;               // 记录app的大小
    bool m_isDraging;                // 记录是否在拖拽

    // widgets
    QBoxLayout *m_mainPanelLayout;

    QWidget *m_fixedAreaWidget;      // 固定区域
    QBoxLayout *m_fixedAreaLayout;   //
    QLabel *m_fixedSpliter;          // 固定区域与应用区域间的分割线
    QWidget *m_appAreaWidget;        // 应用区域
    QWidget *m_appAreaSonWidget;     // 子应用区域，所在位置根据显示模式手动指定
    QBoxLayout *m_appAreaSonLayout;  //
    QLabel *m_appSpliter;            // 应用区域与托盘区域间的分割线

    QWidget *m_appOverflowWidget;       // 溢出区域
    QBoxLayout *m_appOverflowLayout;    // 存放溢出按键和溢出区最近使用的应用
    OverflowItem *m_overflowItem;       // 溢出按键
    AppItem *m_overflowRecentlyUsedApp; // 溢出区最近使用的应用
    QLabel *m_overflowSpliter;          // 溢出区域和应用区域分割线
    DBusDock *m_overflowAppInter;       // 溢出区最近使用的应用dbus接口
    int m_overflowPopMargin;            // 溢出区弹窗左右间距
    QList<QPair<QString, int>> m_overflowAppOpenRecord; // 溢出区应用打开记录，应用Id和布局中对应的索引值

    QWidget *m_trayAreaWidget;       // 托盘区域
    QBoxLayout *m_trayAreaLayout;    //
    QLabel *m_traySpliter;           // 托盘区域与插件区域间的分割线
    QWidget *m_pluginAreaWidget;     // 插件区域
    QBoxLayout *m_pluginLayout;      //
    DesktopWidget *m_desktopWidget;  // 桌面预览区域

    Position m_position;
    QPointer<PlaceholderItem> m_placeholderItem;
    QString m_draggingMimeKey;
    AppDragWidget *m_appDragWidget;
    DisplayMode m_dislayMode;
    QPoint m_mousePressPos;
    TrayPluginItem *m_tray;
    int m_dragIndex;  // 记录应用区域被拖拽图标的位置

    PluginsItem *m_trashItem;  // 垃圾箱插件（需要特殊处理一下）
    QTimer *m_updateDelay;       //
};

#endif  // MAINPANELCONTROL_H
