// SPDX-FileCopyrightText: 2011 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef SYSTEMTRAYITEM_H
#define SYSTEMTRAYITEM_H

#include "constants.h"
#include "../abstracttraywidget.h"
#include "util/dockpopupwindow.h"
#include "pluginsiteminterface.h"

#include <QGestureEvent>
#include <QMenu>

class QGSettings;
class Menu;
class SystemTrayItem : public AbstractTrayWidget
{
    Q_OBJECT

public:
    SystemTrayItem(PluginsItemInterface* const pluginInter, const QString &itemKey, QWidget *parent = nullptr);
    virtual ~SystemTrayItem();

public:
    QString itemKeyForConfig() override;
    void updateIcon() override;
    void sendClick(uint8_t mouseButton, int x, int y) override;
    inline TrayType trayType() const override { return TrayType::SystemTray; }

    QWidget *trayTipsWidget();
    QWidget *trayPopupApplet();
    const QString trayClickCommand();
    const QString contextMenu() const;
    void invokedMenuItem(const QString &menuId, const bool checked);

    static void setDockPostion(const Dock::Position pos) { DockPosition = pos; }

    QWidget *centralWidget() const;
    void detachPluginWidget();
    void showContextMenu();

    void showPopupApplet(QWidget * const applet);
    void hidePopup();

signals:
    void itemVisibleChanged(bool visible);

protected:
    bool event(QEvent *event) override;
    void enterEvent(QEvent *event) override;
    void leaveEvent(QEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void showEvent(QShowEvent* event) override;
    bool eventFilter(QObject *watched, QEvent *event) override;

protected:
    const QPoint popupMarkPoint() const;
    const QPoint topleftPoint() const;

    void hideNonModel();
    void popupWindowAccept();

    virtual void showPopupWindow(QWidget * const content, const bool model = false);
    virtual void showHoverTips();

    bool checkAndResetTapHoldGestureState();
    virtual void gestureEvent(QGestureEvent *event);

protected Q_SLOTS:
    void onContextMenuAccepted();

private:
    void updatePopupPosition();
    void onGSettingsChanged(const QString &key);
    bool checkGSettingsControl() const;
    void menuActionClicked(QAction *action);

private:
    bool m_popupShown;
    bool m_tapAndHold;
    QMenu m_contextMenu;

    PluginsItemInterface* m_pluginInter;
    QWidget *m_centralWidget;

    QTimer *m_popupTipsDelayTimer;
    QTimer *m_popupAdjustDelayTimer;

    QPointer<QWidget> m_lastPopupWidget;
    QString m_itemKey;

    class PopupWindowManager;

    static Dock::Position DockPosition;
    static QPointer<PopupWindowManager> PopupWindow;
    const QGSettings* m_gsettings;

    QWidget *m_mainWindow;
};

class SystemTrayItem::PopupWindowManager : public QObject
{
    Q_OBJECT
public:
    PopupWindowManager(QWidget *parent = nullptr);
    ~PopupWindowManager();

    inline DockPopupWindow *appletPopup() const;
    inline DockPopupWindow *toAppletPopup(SystemTrayItem *item, const QSize &contentSize);
    inline DockPopupWindow *toTipsPopup(SystemTrayItem *item);
    inline DockPopupWindow *data() const;
    bool model() const;
    void hide() const;
    void accept() const;
    bool isVisible() const;
    QWidget *getContent() const;
    void show(const QPoint &pos, const bool model = false);
    void disconnectAccept(SystemTrayItem *item) const;
    void connectAccept(SystemTrayItem *item) const;

public Q_SLOTS:
    void hidePopup(bool visible);

private:
    DockPopupWindow *createPopupWindow(bool noFocus) const;
    inline DockPopupWindow *toPopup(DockPopupWindow *popup, SystemTrayItem *item);
    inline DockPopupWindow *toAppletPopup(DockPopupWindow *popup, SystemTrayItem *item, const QSize &contentSize);

private:
    DockPopupWindow *m_appletPopup;
    DockPopupWindow *m_tipsPopup;
    DockPopupWindow *m_currentPopup;
};

#endif // SYSTEMTRAYITEM_H
